@@ -34,19 +34,24 @@
 #include "arrow/testing/random.h"
 #include "arrow/type.h"
 #include "arrow/util/future.h"
+#include "arrow/util/secure_string.h"
 #include "arrow/util/thread_pool.h"
 #include "parquet/arrow/reader.h"
 #include "parquet/encryption/crypto_factory.h"
-#include "parquet/encryption/encryption_internal.h"
+#include "parquet/encryption/encryption_internal.h"  // for EnsureBackendInitialized
 #include "parquet/encryption/kms_client.h"
 #include "parquet/encryption/test_in_memory_kms.h"
 
-constexpr std::string_view kFooterKeyMasterKey = "0123456789012345";
+using arrow::util::SecureString;
+
+const SecureString kFooterKeyMasterKey("0123456789012345");
 constexpr std::string_view kFooterKeyMasterKeyId = "footer_key";
 constexpr std::string_view kFooterKeyName = "footer_key";
-constexpr std::string_view kColumnMasterKey = "1234567890123450";
+
+const SecureString kColumnMasterKey("1234567890123450");
 constexpr std::string_view kColumnMasterKeyId = "col_key";
-constexpr std::string_view kColumnKeyMapping = "col_key: a";
+constexpr std::string_view kColumnName = "a";
+
 constexpr std::string_view kBaseDir = "";
 
 using arrow::internal::checked_pointer_cast;
@@ -58,6 +63,11 @@ struct EncryptionTestParam {
   bool uniform_encryption;  // false is using per-column keys
   bool concurrently;
   bool use_crypto_factory;
+
+  EncryptionTestParam(bool uniform_encryption, bool concurrently, bool use_crypto_factory)
+      : uniform_encryption(uniform_encryption),
+        concurrently(concurrently),
+        use_crypto_factory(use_crypto_factory) {}
 };
 
 std::ostream& operator<<(std::ostream& os, const EncryptionTestParam& param) {
@@ -74,7 +84,9 @@ const auto kAllParamValues = ::testing::Values(
     EncryptionTestParam{false, true, true}, EncryptionTestParam{true, true, true});
 
 // Base class to test writing and reading encrypted dataset.
-class DatasetEncryptionTestBase : public testing::TestWithParam<EncryptionTestParam> {
+template <typename T,
+          typename = typename std::enable_if_t<std::is_base_of_v<EncryptionTestParam, T>>>
+class DatasetEncryptionTestBase : public testing::TestWithParam<T> {
  public:
 #ifdef ARROW_VALGRIND
   static constexpr int kConcurrentIterations = 4;
@@ -95,30 +107,30 @@ class DatasetEncryptionTestBase : public testing::TestWithParam<EncryptionTestPa
 #endif
 
     // Creates a mock file system using the current time point.
-    EXPECT_OK_AND_ASSIGN(file_system_, fs::internal::MockFileSystem::Make(
-                                           std::chrono::system_clock::now(), {}));
-    ASSERT_OK(file_system_->CreateDir(std::string(kBaseDir)));
+    EXPECT_OK_AND_ASSIGN(
+        file_system_,
+        fs::internal::MockFileSystem::Make(
+            std::chrono::system_clock::now(),
+            {fs::FileInfo(std::string(kBaseDir), fs::FileType::Directory)}));
 
     // Init dataset and partitioning.
     ASSERT_NO_FATAL_FAILURE(PrepareTableAndPartitioning());
     ASSERT_OK_AND_ASSIGN(expected_table_, table_->CombineChunks());
-    ASSERT_OK_AND_ASSIGN(expected_table_, SortTable(expected_table_));
 
     // Prepare encryption properties.
-    std::unordered_map<std::string, std::string> key_map;
-    key_map.emplace(kColumnMasterKeyId, kColumnMasterKey);
-    key_map.emplace(kFooterKeyMasterKeyId, kFooterKeyMasterKey);
+    key_map_.emplace(kColumnMasterKeyId, kColumnMasterKey);
+    key_map_.emplace(kFooterKeyMasterKeyId, kFooterKeyMasterKey);
 
     auto file_format = std::make_shared<ParquetFileFormat>();
     auto parquet_file_write_options =
         checked_pointer_cast<ParquetFileWriteOptions>(file_format->DefaultWriteOptions());
 
-    if (GetParam().use_crypto_factory) {
+    if (this->GetParam().use_crypto_factory) {
       // Configure encryption keys via crypto factory.
       crypto_factory_ = std::make_shared<parquet::encryption::CryptoFactory>();
       auto kms_client_factory =
           std::make_shared<parquet::encryption::TestOnlyInMemoryKmsClientFactory>(
-              /*wrap_locally=*/true, key_map);
+              /*wrap_locally=*/true, key_map_);
       crypto_factory_->RegisterKmsClientFactory(std::move(kms_client_factory));
       kms_connection_config_ =
           std::make_shared<parquet::encryption::KmsConnectionConfig>();
@@ -127,9 +139,11 @@ class DatasetEncryptionTestBase : public testing::TestWithParam<EncryptionTestPa
       auto encryption_config =
           std::make_shared<parquet::encryption::EncryptionConfiguration>(
               std::string(kFooterKeyName));
-      encryption_config->uniform_encryption = GetParam().uniform_encryption;
-      if (!GetParam().uniform_encryption) {
-        encryption_config->column_keys = kColumnKeyMapping;
+      encryption_config->uniform_encryption = this->GetParam().uniform_encryption;
+      if (!this->GetParam().uniform_encryption) {
+        std::stringstream column_key;
+        column_key << kColumnMasterKeyId << ": " << ColumnName();
+        encryption_config->column_keys = column_key.str();
       }
 
       auto parquet_encryption_config = std::make_shared<ParquetEncryptionConfig>();
@@ -142,10 +156,10 @@ class DatasetEncryptionTestBase : public testing::TestWithParam<EncryptionTestPa
     } else {
       // Configure encryption keys via writer options / file encryption properties.
       // non-uniform encryption not support by test
-      ASSERT_TRUE(GetParam().uniform_encryption);
+      ASSERT_TRUE(this->GetParam().uniform_encryption);
       auto file_encryption_properties =
           std::make_unique<parquet::FileEncryptionProperties::Builder>(
-              std::string(kFooterKeyMasterKey))
+              kFooterKeyMasterKey)
               ->build();
       auto writer_properties = std::make_unique<parquet::WriterProperties::Builder>()
                                    ->encryption(file_encryption_properties)
@@ -156,10 +170,10 @@ class DatasetEncryptionTestBase : public testing::TestWithParam<EncryptionTestPa
     // Write dataset.
     auto dataset = std::make_shared<InMemoryDataset>(table_);
     EXPECT_OK_AND_ASSIGN(auto scanner_builder, dataset->NewScan());
-    ARROW_EXPECT_OK(scanner_builder->UseThreads(GetParam().concurrently));
+    ARROW_EXPECT_OK(scanner_builder->UseThreads(this->GetParam().concurrently));
     EXPECT_OK_AND_ASSIGN(auto scanner, scanner_builder->Finish());
 
-    if (GetParam().concurrently) {
+    if (this->GetParam().concurrently) {
       // Have a notable number of threads to exhibit multi-threading issues
       ASSERT_OK_AND_ASSIGN(auto pool, arrow::internal::ThreadPool::Make(16));
       std::vector<Future<>> futures;
@@ -172,6 +186,7 @@ class DatasetEncryptionTestBase : public testing::TestWithParam<EncryptionTestPa
         write_options.base_dir = "thread-" + std::to_string(i);
         write_options.partitioning = partitioning_;
         write_options.basename_template = "part{i}.parquet";
+        write_options.preserve_order = true;
         futures.push_back(
             DeferNotOk(pool->Submit(FileSystemDataset::Write, write_options, scanner)));
       }
@@ -192,9 +207,43 @@ class DatasetEncryptionTestBase : public testing::TestWithParam<EncryptionTestPa
   }
 
   virtual void PrepareTableAndPartitioning() = 0;
+  virtual std::string_view ColumnName() { return kColumnName; }
 
   Result<std::shared_ptr<Dataset>> OpenDataset(
-      std::string_view base_dir, const std::shared_ptr<ParquetFileFormat>& file_format) {
+      std::string_view base_dir,
+      const std::shared_ptr<parquet::encryption::CryptoFactory>& crypto_factory,
+      const std::unordered_map<std::string, SecureString>& key_map) const {
+    // make sure these keys are served by the KMS
+    parquet::encryption::TestOnlyLocalWrapInMemoryKms::InitializeMasterKeys(key_map);
+    parquet::encryption::TestOnlyInServerWrapKms::InitializeMasterKeys(key_map);
+
+    // Set scan options.
+    auto parquet_scan_options = std::make_shared<ParquetFragmentScanOptions>();
+
+    if (this->GetParam().use_crypto_factory) {
+      // Configure decryption keys via crypto factory.
+      auto decryption_config =
+          std::make_shared<parquet::encryption::DecryptionConfiguration>();
+      auto parquet_decryption_config = std::make_shared<ParquetDecryptionConfig>();
+      parquet_decryption_config->crypto_factory = crypto_factory;
+      parquet_decryption_config->kms_connection_config = kms_connection_config_;
+      parquet_decryption_config->decryption_config = std::move(decryption_config);
+
+      parquet_scan_options->parquet_decryption_config =
+          std::move(parquet_decryption_config);
+    } else {
+      // Configure decryption keys via reader properties / file decryption properties.
+      auto file_decryption_properties =
+          std::make_unique<parquet::FileDecryptionProperties::Builder>()
+              ->footer_key(kFooterKeyMasterKey)
+              ->build();
+      parquet_scan_options->reader_properties->file_decryption_properties(
+          file_decryption_properties);
+    }
+
+    auto file_format = std::make_shared<ParquetFileFormat>();
+    file_format->default_fragment_scan_options = std::move(parquet_scan_options);
+
     // Get FileInfo objects for all files under the base directory
     fs::FileSelector selector;
     selector.base_dir = base_dir;
@@ -212,36 +261,10 @@ class DatasetEncryptionTestBase : public testing::TestWithParam<EncryptionTestPa
   }
 
   void TestScanDataset() {
-    // Set scan options.
-    auto parquet_scan_options = std::make_shared<ParquetFragmentScanOptions>();
-
-    if (GetParam().use_crypto_factory) {
-      // Configure decryption keys via crypto factory.
-      auto decryption_config =
-          std::make_shared<parquet::encryption::DecryptionConfiguration>();
-      auto parquet_decryption_config = std::make_shared<ParquetDecryptionConfig>();
-      parquet_decryption_config->crypto_factory = crypto_factory_;
-      parquet_decryption_config->kms_connection_config = kms_connection_config_;
-      parquet_decryption_config->decryption_config = std::move(decryption_config);
-
-      parquet_scan_options->parquet_decryption_config =
-          std::move(parquet_decryption_config);
-    } else {
-      // Configure decryption keys via reader properties / file decryption properties.
-      auto file_decryption_properties =
-          std::make_unique<parquet::FileDecryptionProperties::Builder>()
-              ->footer_key(std::string(kFooterKeyMasterKey))
-              ->build();
-      parquet_scan_options->reader_properties->file_decryption_properties(
-          file_decryption_properties);
-    }
-
-    auto file_format = std::make_shared<ParquetFileFormat>();
-    file_format->default_fragment_scan_options = std::move(parquet_scan_options);
-
-    if (GetParam().concurrently) {
+    if (this->GetParam().concurrently) {
       // Create the dataset
-      ASSERT_OK_AND_ASSIGN(auto dataset, OpenDataset("thread-1", file_format));
+      ASSERT_OK_AND_ASSIGN(auto dataset,
+                           OpenDataset("thread-1", crypto_factory_, key_map_));
 
       // Have a notable number of threads to exhibit multi-threading issues
       ASSERT_OK_AND_ASSIGN(auto pool, arrow::internal::ThreadPool::Make(16));
@@ -260,14 +283,15 @@ class DatasetEncryptionTestBase : public testing::TestWithParam<EncryptionTestPa
 
       // Finally check datasets written by all other threads are as expected
       for (int i = 2; i <= kConcurrentIterations; ++i) {
-        ASSERT_OK_AND_ASSIGN(dataset,
-                             OpenDataset("thread-" + std::to_string(i), file_format));
+        ASSERT_OK_AND_ASSIGN(dataset, OpenDataset("thread-" + std::to_string(i),
+                                                  crypto_factory_, key_map_));
         ASSERT_OK_AND_ASSIGN(auto read_table, ReadDataset(dataset));
         CheckDatasetResults(read_table);
       }
     } else {
       // Create the dataset
-      ASSERT_OK_AND_ASSIGN(auto dataset, OpenDataset(kBaseDir, file_format));
+      ASSERT_OK_AND_ASSIGN(auto dataset,
+                           OpenDataset(kBaseDir, crypto_factory_, key_map_));
 
       // Reuse the dataset above to scan it twice to make sure decryption works correctly.
       for (int i = 0; i < 2; ++i) {
@@ -282,44 +306,37 @@ class DatasetEncryptionTestBase : public testing::TestWithParam<EncryptionTestPa
     // Read dataset into table
     ARROW_ASSIGN_OR_RAISE(auto scanner_builder, dataset->NewScan());
     ARROW_ASSIGN_OR_RAISE(auto scanner, scanner_builder->Finish());
-    ARROW_EXPECT_OK(scanner_builder->UseThreads(GetParam().concurrently));
+    ARROW_EXPECT_OK(scanner_builder->UseThreads(
+        testing::WithParamInterface<T>::GetParam().concurrently));
     return scanner->ToTable();
   }
 
   void CheckDatasetResults(const std::shared_ptr<Table>& table) {
     ASSERT_OK(table->ValidateFull());
-    // Make results comparable despite ordering and chunking differences
+    // Make results comparable despite chunking differences
     ASSERT_OK_AND_ASSIGN(auto combined_table, table->CombineChunks());
-    ASSERT_OK_AND_ASSIGN(auto sorted_table, SortTable(combined_table));
-    AssertTablesEqual(*sorted_table, *expected_table_);
-  }
-
-  // Sort table for comparability of dataset read results, which may be unordered.
-  // This relies on column "a" having statistically unique values.
-  Result<std::shared_ptr<Table>> SortTable(const std::shared_ptr<Table>& table) {
-    compute::SortOptions options({compute::SortKey("a")});
-    ARROW_ASSIGN_OR_RAISE(auto indices, compute::SortIndices(table, options));
-    ARROW_ASSIGN_OR_RAISE(auto sorted, compute::Take(table, indices));
-    EXPECT_EQ(sorted.kind(), Datum::TABLE);
-    return sorted.table();
+    AssertTablesEqual(*combined_table, *expected_table_);
   }
 
  protected:
-  std::string base_dir_ = GetParam().concurrently ? "thread-1" : std::string(kBaseDir);
+  std::string base_dir_ =
+      this->GetParam().concurrently ? "thread-1" : std::string(kBaseDir);
   std::shared_ptr<fs::FileSystem> file_system_;
   std::shared_ptr<Table> table_, expected_table_;
   std::shared_ptr<Partitioning> partitioning_;
   std::shared_ptr<parquet::encryption::CryptoFactory> crypto_factory_;
   std::shared_ptr<parquet::encryption::KmsConnectionConfig> kms_connection_config_;
+  std::unordered_map<std::string, SecureString> key_map_;
 };
 
-class DatasetEncryptionTest : public DatasetEncryptionTestBase {
+class DatasetEncryptionTest : public DatasetEncryptionTestBase<EncryptionTestParam> {
  public:
   // The dataset is partitioned using a Hive partitioning scheme.
   void PrepareTableAndPartitioning() override {
     // Prepare table data.
-    auto table_schema = schema({field("a", int64()), field("c", int64()),
-                                field("e", int64()), field("part", utf8())});
+    auto table_schema =
+        schema({field(std::string(kColumnName), int64()), field("c", int64()),
+                field("e", int64()), field("part", utf8())});
     table_ = TableFromJSON(table_schema, {R"([
                           [ 0, 9, 1, "a" ],
                           [ 1, 8, 2, "a" ],
@@ -370,7 +387,7 @@ TEST_P(DatasetEncryptionTest, ReadSingleFile) {
     // Configure decryption keys via file decryption properties with static footer key.
     file_decryption_properties =
         std::make_unique<parquet::FileDecryptionProperties::Builder>()
-            ->footer_key(std::string(kFooterKeyMasterKey))
+            ->footer_key(kFooterKeyMasterKey)
             ->build();
   }
   auto reader_properties = parquet::default_reader_properties();
@@ -393,9 +410,191 @@ TEST_P(DatasetEncryptionTest, ReadSingleFile) {
 
 INSTANTIATE_TEST_SUITE_P(DatasetEncryptionTest, DatasetEncryptionTest, kAllParamValues);
 
+struct NestedFieldsEncryptionTestParam : EncryptionTestParam {
+  std::string column_name;
+
+  explicit NestedFieldsEncryptionTestParam(std::string column_name)
+      : EncryptionTestParam(false, false, true), column_name(std::move(column_name)) {}
+};
+
+std::ostream& operator<<(std::ostream& os, const NestedFieldsEncryptionTestParam& param) {
+  os << param.column_name;
+  return os;
+}
+
+class NestedFieldsEncryptionTest
+    : public DatasetEncryptionTestBase<NestedFieldsEncryptionTestParam> {
+ public:
+  NestedFieldsEncryptionTest() : id_data_(ArrayFromJSON(int8(), "[1,2,3]")) {}
+
+  void SetUp() override {
+    DatasetEncryptionTestBase::SetUp();
+
+    // deliberately mis-configured column key
+    key_map_with_wrong_column_key_.emplace(kColumnMasterKeyId, kFooterKeyMasterKey);
+    key_map_with_wrong_column_key_.emplace(kFooterKeyMasterKeyId, kFooterKeyMasterKey);
+
+    crypto_factory_with_wrong_column_key_ =
+        std::make_shared<parquet::encryption::CryptoFactory>();
+    auto kms_client_factory =
+        std::make_shared<parquet::encryption::TestOnlyInMemoryKmsClientFactory>(
+            /*wrap_locally=*/true, key_map_with_wrong_column_key_);
+    crypto_factory_with_wrong_column_key_->RegisterKmsClientFactory(
+        std::move(kms_client_factory));
+  }
+
+  // The dataset is partitioned using a directory partitioning scheme.
+  void PrepareTableAndPartitioning() override {
+    // Prepare table and partitioning.
+    auto table_schema = schema({field("id", int8()), field("a", column_type_)});
+    table_ = arrow::Table::Make(table_schema, {id_data_, column_data_});
+    partitioning_ = std::make_shared<dataset::DirectoryPartitioning>(arrow::schema({}));
+  }
+
+  static const NestedFieldsEncryptionTestParam& GetParam() {
+    return WithParamInterface::GetParam();
+  }
+
+  std::string_view ColumnName() override { return GetParam().column_name; }
+
+  void TestDatasetColumnEncryption() {
+    // Read the dataset with wrong column key
+    ASSERT_OK_AND_ASSIGN(auto dataset,
+                         OpenDataset(kBaseDir, crypto_factory_with_wrong_column_key_,
+                                     key_map_with_wrong_column_key_));
+
+    // Reuse the dataset above to scan it twice to make sure decryption works correctly.
+    for (size_t i = 0; i < 2; ++i) {
+      // Read the non-encrypted id column into table
+      ASSERT_OK_AND_ASSIGN(auto id_scanner_builder, dataset->NewScan());
+      ASSERT_OK(id_scanner_builder->Project({"id"}));
+      ASSERT_OK_AND_ASSIGN(auto id_scanner, id_scanner_builder->Finish());
+      ASSERT_OK_AND_ASSIGN(auto read_table, id_scanner->ToTable());
+
+      // Verify the column was read correctly
+      ASSERT_OK_AND_ASSIGN(auto combined_table, read_table->CombineChunks());
+      ASSERT_OK(combined_table->ValidateFull());
+      ASSERT_OK_AND_ASSIGN(auto expected_table, table_->RemoveColumn(1));
+      AssertTablesEqual(*combined_table, *expected_table);
+
+      // Read the encrypted column 'a' into table
+      ASSERT_OK_AND_ASSIGN(auto a_scanner_builder, dataset->NewScan());
+      ASSERT_OK(a_scanner_builder->Project({"a"}));
+      ASSERT_OK_AND_ASSIGN(auto a_scanner, a_scanner_builder->Finish());
+      ASSERT_RAISES_WITH_MESSAGE(IOError, "IOError: Failed decryption finalization",
+                                 a_scanner->ToTable());
+    }
+  }
+
+ protected:
+  std::shared_ptr<DataType> column_type_;
+  std::shared_ptr<Array> id_data_;
+  std::shared_ptr<Array> column_data_;
+  std::shared_ptr<parquet::encryption::CryptoFactory>
+      crypto_factory_with_wrong_column_key_;
+  std::unordered_map<std::string, SecureString> key_map_with_wrong_column_key_;
+};
+
+class ListFieldEncryptionTest : public NestedFieldsEncryptionTest {
+ public:
+  ListFieldEncryptionTest() {
+    column_type_ = list(int32());
+    column_data_ = ArrayFromJSON(column_type_, "[[1, 2, 3], [4, 5], [6]]");
+  }
+};
+
+class MapFieldEncryptionTest : public NestedFieldsEncryptionTest {
+ public:
+  MapFieldEncryptionTest() {
+    column_type_ = map(utf8(), int32());
+    column_data_ =
+        ArrayFromJSON(column_type_, R"([[["one", 1]], [["two", 2]], [["three", 3]]])");
+  }
+};
+
+class StructFieldEncryptionTest : public NestedFieldsEncryptionTest {
+ public:
+  StructFieldEncryptionTest() {
+    column_type_ = struct_({field("f1", int32()), field("f2", utf8())});
+    column_data_ = ArrayFromJSON(
+        column_type_,
+        R"([{"f1":1, "f2":"one"}, {"f1":2, "f2":"two"}, {"f1":3, "f2":"three"}])");
+  }
+};
+
+class DeepNestedFieldEncryptionTest : public NestedFieldsEncryptionTest {
+ public:
+  DeepNestedFieldEncryptionTest() {
+    auto struct_type = struct_({field("f1", int32()), field("f2", utf8())});
+    auto map_type = map(int32(), struct_type);
+    auto list_type = list(map_type);
+
+    column_type_ = list_type;
+    column_data_ = ArrayFromJSON(column_type_, R"([
+      [
+        [[1, {"f1":1, "f2":"one"}], [2, {"f1":2, "f2":"two"}]],
+        [[3, {"f1":3, "f2":"three"}]]
+      ],
+      [
+        [[4, {"f1":4, "f2":"fur"}]]
+      ],
+      []
+    ])");
+  }
+};
+
+const auto kListFieldEncryptionTestParamValues =
+    ::testing::Values(NestedFieldsEncryptionTestParam("a"),
+                      NestedFieldsEncryptionTestParam("a.list.element"));
+
+const auto kMapFieldEncryptionTestParamValues =
+    ::testing::Values(NestedFieldsEncryptionTestParam("a"),
+                      NestedFieldsEncryptionTestParam("a.key_value.key"),
+                      NestedFieldsEncryptionTestParam("a.key_value.value"));
+
+const auto kStructFieldEncryptionTestParamValues = ::testing::Values(
+    NestedFieldsEncryptionTestParam("a"), NestedFieldsEncryptionTestParam("a.f1"),
+    NestedFieldsEncryptionTestParam("a.f2"));
+
+const auto kDeepNestedFieldEncryptionTestParamValues = ::testing::Values(
+    NestedFieldsEncryptionTestParam("a"),
+    NestedFieldsEncryptionTestParam("a.list.element.key_value.key"),
+    NestedFieldsEncryptionTestParam("a.list.element.key_value.value.f1"),
+    NestedFieldsEncryptionTestParam("a.list.element.key_value.value.f2"));
+
+TEST_P(ListFieldEncryptionTest, ColumnKeys) {
+  ASSERT_NO_FATAL_FAILURE(TestScanDataset());
+  ASSERT_NO_FATAL_FAILURE(TestDatasetColumnEncryption());
+}
+
+TEST_P(MapFieldEncryptionTest, ColumnKeys) {
+  ASSERT_NO_FATAL_FAILURE(TestScanDataset());
+  ASSERT_NO_FATAL_FAILURE(TestDatasetColumnEncryption());
+}
+
+TEST_P(StructFieldEncryptionTest, ColumnKeys) {
+  ASSERT_NO_FATAL_FAILURE(TestScanDataset());
+  ASSERT_NO_FATAL_FAILURE(TestDatasetColumnEncryption());
+}
+
+TEST_P(DeepNestedFieldEncryptionTest, ColumnKeys) {
+  ASSERT_NO_FATAL_FAILURE(TestScanDataset());
+  ASSERT_NO_FATAL_FAILURE(TestDatasetColumnEncryption());
+}
+
+// Test writing and reading encrypted nested fields
+INSTANTIATE_TEST_SUITE_P(List, ListFieldEncryptionTest,
+                         kListFieldEncryptionTestParamValues);
+INSTANTIATE_TEST_SUITE_P(Map, MapFieldEncryptionTest, kMapFieldEncryptionTestParamValues);
+INSTANTIATE_TEST_SUITE_P(Struct, StructFieldEncryptionTest,
+                         kStructFieldEncryptionTestParamValues);
+INSTANTIATE_TEST_SUITE_P(DeepNested, DeepNestedFieldEncryptionTest,
+                         kDeepNestedFieldEncryptionTestParamValues);
+
 // GH-39444: This test covers the case where parquet dataset scanner crashes when
 // processing encrypted datasets over 2^15 rows in multi-threaded mode.
-class LargeRowCountEncryptionTest : public DatasetEncryptionTestBase {
+class LargeRowCountEncryptionTest
+    : public DatasetEncryptionTestBase<EncryptionTestParam> {
  public:
   // The dataset is partitioned using a Hive partitioning scheme.
   void PrepareTableAndPartitioning() override {
